/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

'use strict';

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

const Gettext = imports.gettext.domain('gnome-shell-extensions-commute');
const _ = Gettext.gettext;

const Me = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Me.imports.lib;

let gsettings;
let settings;
let generalSettings;
let googleSettings;
let bingSettings;
let settings_indicator;

function init() {
    gsettings = Lib.getSettings(Me);
    generalSettings = {
        providers: {
            type: "e",
            label: _("Route engine"),
            list: [
                { nick: "google", name:_("Google"), id: 0 },
                { nick: "bing",   name:_("Bing"),   id: 1 }
            ]
        },
        origin: {
            type: "s",
            label: _("Origin"),
            help: _("Write the address of your route start")
        },
        destination: {
            type: "s",
            label: _("Destination"),
            help: _("Write the address of your route end")
        },
        indicator_position: {
            type: "e",
            label: _("Indicator position"),
            list: [
                { nick: "center", name: _("Center"), id: 0 },
                { nick: "right",  name: _("Right"),  id: 1 },
                { nick: "left",   name: _("Left"),   id: 2 }
            ]
        },
        refresh_interval: {
            type: "s",
            label: _("Refresh interval in minutes (0 means no autorefresh)."),
            help: _("Autorefresh interval in minutes of the route. If turned off, the information will be refreshed upon button press only.")
        },
        automatic_switch: {
            type: "b",
            label: _("Switch automatically origin with destination at a given time?")
        },
        switch_time: {
            type: "s",
            label: _("Time of day to switch the origin and destination"),
            help: _("Use the format hh:mm to enter the time, default is 12:00!")
        },
        displayforecast_switch: {
            type: "b",
            label: _("Should display traffic forecast vs. real values?")
        }
    };
    googleSettings = {
        google_key: {
            type: "s",
            label: _("API Key"),
            help: _("Get a key from the map provider")
        },
        google_alternatives: {
            type: "b",
            label: _("Allows to request route alternatives")
        },
        units: {
            type: "e",
            label: _("Measurement unit"),
            list: [
                { nick: "metric",   name: _("Metric"),   id: 0 },
                { nick: "imperial", name: _("Imperial"), id: 1 }
            ]
        },
        mode: {
            type: "e",
            label: _("Driving mode"),
            list: [
                { nick: "driving",   name: _("Driving"),   id: 0 },
                { nick: "walking",   name: _("Walking"),   id: 1 },
                { nick: "bicycling", name: _("Bicycling"), id: 2 },
                { nick: "transit",   name: _("Transit"),   id: 3 }
            ]
        },
        transit_mode: {
            type: "e",
            label: _("Transit mode"),
            list: [
                { nick: "none",   name: _("None"),   id: 0 },
                { nick: "bus",    name: _("Bus"),    id: 1 },
                { nick: "subway", name: _("Subway"), id: 2 },
                { nick: "train",  name: _("Train"),  id: 3 },
                { nick: "tram",   name: _("Tram"),   id: 4 },
                { nick: "rail",   name: _("Rail"),   id: 5 }
            ]
        },
        avoid: {
            type: "e",
            label: _("Routes to avoid"),
            list: [
                { nick: "none",     name: _("None"),     id: 0 },
                { nick: "tolls",    name: _("Tolls"),    id: 1 },
                { nick: "highways", name: _("Highways"), id: 2 },
                { nick: "ferries",  name: _("Ferries"),  id: 3 },
                { nick: "indoor",   name: _("Indoor"),   id: 4 }
            ]
        },
        traffic_model: {
            type: "e",
            label: _("Traffic calculation assumptions"),
            list: [
                { nick: "best_guess",  name: _("Best guess"),  id: 0 },
                { nick: "pessimistic", name: _("Pessimistic"), id: 1 },
                { nick: "optimistic",  name: _("Optimistic"),  id: 2 }
            ]
        }
    };
    bingSettings = {
        bing_key: {
            type: "s",
            label: _("API Key"),
            help: _("Get a key from the map provider")
        },
        bing_alternatives: {
            type: "b",
            label: _("Allows to request route alternatives")
        },
        bing_units: {
            type: "e",
            label: _("Measurement unit"),
            list: [
                { nick: "Mile",      name: _("Imperial"),   id: 0 },
                { nick: "Kilometer", name: _("Metric"),     id: 1 }
            ]
        },
        bing_avoid: {
            type: "e",
            label: _("Routes to avoid"),
            list: [
                { nick: "none",              name: _("None"),               id: 0 },
                { nick: "highways",          name: _("Highways"),           id: 1 },
                { nick: "tolls",             name: _("Tolls"),              id: 2 },
                { nick: "minimizeHighways",  name: _("Minimize highways"),  id: 3 },
                { nick: "minimizeTolls",     name: _("minimize tolls"),     id: 4 }
            ]
        },
        bing_travelmode: {
            type: "e",
            label: _("Driving mode"),
            list: [
                { nick: "Driving",   name: _("Driving"),   id: 0 },
                { nick: "Walking",   name: _("Walking"),   id: 1 },
                { nick: "Transit",   name: _("Transit"),   id: 2 }
            ]
        },
        bing_optimize: {
            type: "e",
            label: _("Route calculation optimization"),
            list: [
                { nick: "distance",         name: _("Distance"),           id: 0 },
                { nick: "time",             name: _("Time"),               id: 1 },
                { nick: "timeWithTraffic",  name: _("Time with Traffic"),  id: 2 },
                { nick: "timeAvoidClosure", name: _("Time avoid closure"), id: 3 }
            ]
        }
    };
}

function buildPrefsWidget() {
    let frame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL,
                             border_width: 10});
    let hbox;

    let notebook = new Gtk.Notebook();
    notebook.set_tab_pos(Gtk.PositionType.TOP);
    let generalFrame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL,
                            margin: 20, margin_top: 10});
    let generalPage = new Gtk.Label({label: "General", xalign: 0 });
    notebook.append_page(generalFrame, generalPage);

    let googleFrame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL,
                            margin: 20, margin_top: 10});
    let googlePage = new Gtk.Label({label: "Google", xalign: 0 });
    notebook.append_page(googleFrame, googlePage);

    let bingFrame = new Gtk.Box({orientation: Gtk.Orientation.VERTICAL,
                            margin: 20, margin_top: 10});
    let bingPage = new Gtk.Label({label: "Bing", xalign: 0 });
    notebook.append_page(bingFrame, bingPage);

    notebook.show_all();

    settings = generalSettings;
    for (let setting in settings) {
        hbox = buildHbox(settings, setting);
        generalFrame.add(hbox);
    }
    generalFrame.show_all();

    settings = googleSettings;
    for (let setting in settings) {
        hbox = buildHbox(settings, setting);
        googleFrame.add(hbox);
    }
    googleFrame.show_all();

    settings = bingSettings;
    for (let setting in settings) {
        hbox = buildHbox(settings, setting);
        bingFrame.add(hbox);
    }
    bingFrame.show_all();

    return notebook;
}

function buildHbox(settings, setting) {
    let hbox;

    if (settings[setting].type == 's')
        hbox = createStringSetting(settings, setting);
    if (settings[setting].type == "b")
        hbox = createBoolSetting(settings, setting);
    if (settings[setting].type == "e")
        hbox = createEnumSetting(settings, setting);

    return hbox;
}

function createEnumSetting(settings, setting) {

    let hbox = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL,
                            margin_top: 5});

    let setting_label = new Gtk.Label({label: settings[setting].label,
                                       xalign: 0 });

    let model = new Gtk.ListStore();
    model.set_column_types([GObject.TYPE_INT, GObject.TYPE_STRING]);
    let setting_enum = new Gtk.ComboBox({model: model});
    setting_enum.get_style_context().add_class(Gtk.STYLE_CLASS_RAISED);
    let renderer = new Gtk.CellRendererText();
    setting_enum.pack_start(renderer, true);
    setting_enum.add_attribute(renderer, 'text', 1);

    for (let i=0; i<settings[setting].list.length; i++) {
        let item = settings[setting].list[i];
        let iter = model.append();
        model.set(iter, [0, 1], [item.id, item.name]);
        if (item.id == gsettings.get_enum(setting.replace('_', '-'))) {
            setting_enum.set_active(item.id);
        }
    }

    setting_enum.connect('changed', function(entry) {
        let [success, iter] = setting_enum.get_active_iter();
        if (!success)
            return;

        let id = model.get_value(iter, 0);
        gsettings.set_enum(setting.replace('_', '-'), id);
    });

    if (settings[setting].help) {
        setting_label.set_tooltip_text(settings[setting].help);
        setting_enum.set_tooltip_text(settings[setting].help);
    }

    hbox.pack_start(setting_label, true, true, 0);
    hbox.add(setting_enum);

    return hbox;

}

function createStringSetting(settings, setting) {

    let hbox = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL,
                            margin_top: 5});

    let setting_label = new Gtk.Label({label: settings[setting].label,
                                       xalign: 0 });

    let setting_string = new Gtk.Entry({text: gsettings.get_string(setting.replace('_', '-'))});
    setting_string.set_width_chars(30);
    setting_string.connect('notify::text', function(entry) {
        gsettings.set_string(setting.replace('_', '-'), entry.text);
    });

    if (settings[setting].mode == "passwd") {
        setting_string.set_visibility(false);
    }

    if (settings[setting].help) {
        setting_label.set_tooltip_text(settings[setting].help);
        setting_string.set_tooltip_text(settings[setting].help);
    }

    hbox.pack_start(setting_label, true, true, 0);
    hbox.add(setting_string);

    return hbox;
}

function createBoolSetting(settings, setting) {

    let hbox = new Gtk.Box({orientation: Gtk.Orientation.HORIZONTAL,
                            margin_top: 5});

    let setting_label = new Gtk.Label({label: settings[setting].label,
                                       xalign: 0 });

    let setting_switch = new Gtk.Switch({active: gsettings.get_boolean(setting.replace('_', '-'))});
    setting_switch.connect('notify::active', function(button) {
        gsettings.set_boolean(setting.replace('_', '-'), button.active);
    });

    if (settings[setting].help) {
        setting_label.set_tooltip_text(settings[setting].help);
        setting_switch.set_tooltip_text(settings[setting].help);
    }

    hbox.pack_start(setting_label, true, true, 0);
    hbox.add(setting_switch);

    return hbox;
}

